/******************************************************************************
* Project Name		: BLE_Whitelist
* File Name			: main.c
* Version 			: 1.0
* Device Used		: CY8C4247LQI-BL483
* Hardware          : CY8CKIT-042-BLE
* Software Used		: PSoC Creator 3.1 CP1
* Compiler    		: ARM GCC 4.8.4, ARM RVDS Generic, ARM MDK Generic
* Owner				: MADY
*
********************************************************************************
* Copyright (2014-15), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
/******************************************************************************
*                           THEORY OF OPERATION
*******************************************************************************
*Whitelist functionality in Bluetooth allows a device to filter out other devices 
*trying to scan or connect with the device. Only the devices added to whitelist 
*will be given the privilege of scanning or connection or both (based on the filter
*policy selected). In this example, the BLE Component is programmed as a Peripheral
*and a Server, which has the Whitelist setting as “Scan Request Whitelist; Connect
*Request Whitelist”. This means that, only the central devices that are added to our
*peripheral’s Whitelist will be able to receive the scan response packet and establish
*a connection with it. UART Terminal (Baud Rate: 115200) is used to provide inputs in 
*this example. The user can manually add any device to the whitelist and can remove a 
*device from the whitelist.
******************************************************************************/
#include <project.h>
#include "stdio.h"


#include "LED.h"

void StackEventHandler(uint32 event,void *eventParam);
//Structure containing address of device 
CYBLE_GAP_BD_ADDR_T     whitelistdeviceaddress;
//Array which is used to store the list of devices added.
CYBLE_GAP_BD_ADDR_T     whitelistdeviceaddressBackup[8];
uint8                   Index;         //Index of devices added to Whitelist

uint32                  UartRxDataSim;//Character Input from the UART Terminal
uint8                   Count;        /*Count of the Characters from UART while 
                                        receiving Address, for adding to whitelist*/
uint8                   RemoveIndex;  //Index of the device to be removed from whitelist
uint8                   AddrNibble;     //Actual value derived from INPUT ASCII value

uint8                   AddRequest;   /*FLag which is set when requeust to add a
                                        device to whitelist is received */   
uint8                   DelRequest;   /* FLag which is set when requeust to remove
                                         a device from whitelist is received */
CYBLE_API_RESULT_T 		apiResult;
char MyBuffer[256];
uint8 bdHandle;
CYBLE_CONN_HANDLE_T connHandle;
CYBLE_GAPP_DISC_MODE_INFO_T Discovery_Parameters;
CYBLE_GAP_BD_ADDR_T BD_Peripheral;
    
/*******************************************************************************
* Function Name: StackEventHandler
********************************************************************************
* Summary:
*        Call back event function to handle various events from BLE stack
*
* Parameters:
*  event:		event returned
*  eventParam:	link to value of the event parameter returned
*
* Return:
*  void
*
*******************************************************************************/

void StackEventHandler(uint32 event,void *eventParam)
{
    switch(event)
	{
        
		case CYBLE_EVT_STACK_ON:
            //Starting Advertisement as soon as Stack is ON
             apiResult = CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            if (apiResult == CYBLE_ERROR_OK)
            {
                printf ("Starting Advertisement\r\n");
            }
            else
            {
                printf ("Error Start Adv %d\r\n",apiResult);
            }
            apiResult = CyBle_SetAppEventMask (0xFFFF);
            break;
            
        case CYBLE_EVT_TIMEOUT:
         if( CYBLE_GAP_ADV_MODE_TO ==*(uint16*) eventParam)
            {
                printf ("Advertisement TimedOut\r\n");
                apiResult = CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
                if (apiResult != CYBLE_ERROR_OK)
                {
                    printf ("\nRestarting  Advertisement\r\n");
                }
            }

		case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
            UART_PutString ("\nCYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP\r\n");
             IsInWhiteList();
            break;            
            
		case CYBLE_EVT_GATT_DISCONNECT_IND:
			/* Red LED Glows when device is disconnected */
            RED_LED_ON ();
            break;
            
        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            printf ("Disconnected \r\n");
            // Starting Advertisent again when there is disconnection
            apiResult = CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            if (apiResult != CYBLE_ERROR_OK)
            {
                UART_PutString ("\nRestarting  Advertisement\r\n");
            }
            cyBle_discoveryModeInfo.advParam->advFilterPolicy = CYBLE_GAPP_SCAN_WHITELIST_CONN_ANY;
            apiResult = CyBle_GappEnterDiscoveryMode(&cyBle_discoveryModeInfo);
            if (apiResult == CYBLE_ERROR_OK)
            {
                UART_PutString ("\nCyBle_GappEnterDiscoveryMode OK\r\n");
            }     

            break;
        
        case CYBLE_EVT_GATT_CONNECT_IND:
            connHandle = *(CYBLE_CONN_HANDLE_T *)eventParam;
            UART_PutString("Device in Connect Ind\r\n");
            break;

        case CYBLE_EVT_GAP_SCAN_REQ_RECVD:
            UART_PutString("Receiving CYBLE_EVT_GAP_SCAN_REQ_RECVD\r\n");
            break;
            
        case CYBLE_EVT_GAPC_DIRECT_ADV_REPORT:
            UART_PutString("Entering CYBLE_EVT_GAPC_DIRECT_ADV_REPORT\r\n");
            break;    
			
		case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            printf ("\n\r Connection Established \r\n");
            //Blue LED glows when device is connected
            BLUE_LED_ON ();
            CyBle_GapGetPeerBdAddr(connHandle.bdHandle, &whitelistdeviceaddress);
            sprintf(MyBuffer,"MyPeerAddress %x",whitelistdeviceaddress.bdAddr[0]);
            UART_PutString(MyBuffer);
            sprintf(MyBuffer,"%x",whitelistdeviceaddress.bdAddr[1]);
            UART_PutString(MyBuffer);
            sprintf(MyBuffer,"%x",whitelistdeviceaddress.bdAddr[2]);
            UART_PutString(MyBuffer);
            sprintf(MyBuffer,"%x",whitelistdeviceaddress.bdAddr[3]);
            UART_PutString(MyBuffer);
            sprintf(MyBuffer,"%x",whitelistdeviceaddress.bdAddr[4]);
            UART_PutString(MyBuffer);
            sprintf(MyBuffer,"%x\r\n",whitelistdeviceaddress.bdAddr[5]);
            UART_PutString(MyBuffer);
            apiResult = CyBle_GapAddDeviceToWhiteList(&whitelistdeviceaddress); 
                                
            if(apiResult == CYBLE_ERROR_INVALID_PARAMETER)
  			{
       			UART_PutString ("Adding to Whitelist Failed. Invalid parameter \r\n");
            }
            else if   (apiResult == CYBLE_ERROR_INVALID_OPERATION)
            {
                printf ("Invalid Operation \r\n");
            }
            else if (apiResult == CYBLE_ERROR_INSUFFICIENT_RESOURCES)
            {
                printf ("Adding to Whitelist Failed. List already full \r\n");
            }
            else if (apiResult == CYBLE_ERROR_DEVICE_ALREADY_EXISTS)
            {
                printf ("Device Already exists \r\n");
            }
            else if (apiResult == CYBLE_ERROR_OK)
            {
                //Copying the address to the BackUp Array
                whitelistdeviceaddressBackup[Index] = whitelistdeviceaddress; 
                Index++;
                UART_PutString ("Device Added to WhiteList\r\n");
            }
          
        break;
		
	}
}

uint8 IsInWhiteList (void)
{
 
    uint8 Output=0;
    uint8 count[8];
    int i;
    CYBLE_GAP_BD_ADDR_T ListAddr[8];
    
    CyBle_GapGetDevicesFromWhiteList(count,ListAddr);
    for (i=0;i<8;i++)
    {
        
        sprintf(MyBuffer,"WhiteList[%x] %x",i,ListAddr[i].bdAddr[0]);
        UART_PutString(MyBuffer);
        sprintf(MyBuffer,"%x",ListAddr[i].bdAddr[1]);
        UART_PutString(MyBuffer);
        sprintf(MyBuffer,"%x",ListAddr[i].bdAddr[2]);
        UART_PutString(MyBuffer);
        sprintf(MyBuffer,"%x",ListAddr[i].bdAddr[3]);
        UART_PutString(MyBuffer);
        sprintf(MyBuffer,"%x",ListAddr[i].bdAddr[4]);
        UART_PutString(MyBuffer);
        sprintf(MyBuffer,"%x\r\n",ListAddr[i].bdAddr[5]);
        UART_PutString(MyBuffer);
    }
    
    return Output;
}

/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
*  System entry and execution point for application
*
* Parameters:  
*  None
*
* Return: 
*  int

********************************************************************************/

int main()
{
  
    /* Initializing all the Flags and Indexes to 0 */
    ALL_LED_OFF ();

    CyGlobalIntEnable;  /* Comment this line to disable global interrupts. */
    
    /* Start BLE component and register Event handler function */	
    CyBle_Start(StackEventHandler);

    /* Start UART Component which is used for receiving inputs and Debugging */
    UART_Start();

	UART_PutString("BLE WhiteList Example \r\n");
    
    for(;;)
    {
        //Checks the internal task queue in the BLE Stack
        CyBle_ProcessEvents();
        
    }
}

/* [] END OF FILE */
